package it.polito.bigdata.spark.example;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.sql.types.DataTypes;


public class SparkDriver {
	
	public static void main(String[] args) {

		// The following two lines are used to switch off some verbose log messages
		Logger.getLogger("org").setLevel(Level.OFF);
		Logger.getLogger("akka").setLevel(Level.OFF);


		String housePath, consumptionPath;
		String outputPath1, outputPath2;

		housePath = args[0];
		consumptionPath = args[1];
		outputPath1 = args[2];
		outputPath2 = args[3];

	
		// Create a Spark Session object and set the name of the application
		SparkSession ss = SparkSession.builder().appName("Exam20220202 Spark")
									.master("local").getOrCreate();

		// Define the dataframes associated with the input files
		Dataset<Row> houseDF = ss.read().format("csv")
				.option("header", false).option("inferSchema", true)
				.load(housePath)
				.withColumnRenamed("_c0", "HouseID")
				.withColumnRenamed("_c1", "City")
				.withColumnRenamed("_c2", "Country")
				.withColumnRenamed("_c3", "SizeSQM");

		// Register the temporary table houses 
		houseDF.createOrReplaceTempView("houses");


		Dataset<Row> consumptionDF = ss.read().format("csv")
				.option("header", false).option("inferSchema", true)
				.load(consumptionPath)
				.withColumnRenamed("_c0", "HouseID")
				.withColumnRenamed("_c1", "Date")
				.withColumnRenamed("_c2", "kWh");

		// Register the temporary table consumption 
		consumptionDF.createOrReplaceTempView("consumption");


		// Part 1

		// Register a function that given a date in the format "" returns the year  
		ss.udf().register("YEAR", 
				(String date) -> Integer.parseInt(date.split("/")[0]),
				DataTypes.IntegerType);

		// Consider only the readings associated with the year 2022,
		// compute the average daily consumption for each house in the year 2022,
		// and select the houses with a daily consumption >30
		Dataset<Row> housesHighConsumptionDF = 
			ss.sql("SELECT HouseID "
			  		+ "FROM consumption "
					+ "WHERE YEAR(Date)=2022 "
					+ "GROUP BY HouseID "
					+ "HAVING SUM(kWh)/365>30");
		
		// Register the temporary table housesHighCons 
		housesHighConsumptionDF.createOrReplaceTempView("housesHighCons");


		Dataset<Row> res1DF = 
			ss.sql("SELECT DISTINCT Country "
			  		+ "FROM houses "
					+ "WHERE Country NOT IN ("
					+ " SELECT Country "
					+ " FROM housesHighCons, houses "
					+ " WHERE housesHighCons.HouseID=houses.HouseID)");
		
		res1DF.write()
		.format("csv")
		.option("header", false)
		.save(outputPath1);



		// Part 2
		// keep only the houses for which the total power consumption over 2021 is > threshold kWh
		Dataset<Row> highTotalPowerCons2021DF = 
			ss.sql("SELECT HouseID "
			  		+ "FROM consumption "
					+ "WHERE YEAR(Date)=2021 "
					+ "GROUP BY HouseID "
					+ "HAVING SUM(kWh)>10000");
		
		// Register the temporary table housesHighCons2021 
		highTotalPowerCons2021DF.createOrReplaceTempView("housesHighCons2021");


		// Select for each country the cities with at least 500 houses with high annual power consumption
		Dataset<Row> coutryCityManyHighConsHousesDF = 
			ss.sql("SELECT City, Country "  // Selecting the city is useless. You can select only Country (SELECT Country), but pay attention to keep GROUP BY City, Country
			  		+ "FROM houses, housesHighCons2021 "
					+ "WHERE houses.HouseID=housesHighCons2021.HouseID "
					+ "GROUP BY City, Country "
					+ "HAVING COUNT(*)>500");

		// Register the temporary table citiesWithManyHighConsHouses 
		coutryCityManyHighConsHousesDF.createOrReplaceTempView("citiesWithManyHighConsHouses");

		// Compute for each country the number of cities with at least 500 houses with high annual power consumption
		// We must consider also the countries without cities with at least 500 houses with high annual power consumption
		Dataset<Row> highPwrConsCitiesPerCountryDF = 
			ss.sql("SELECT Country, SUM(CityWithManyHighConsHouses) "
			  		+ "FROM ( (SELECT Country, 1 as CityWithManyHighConsHouses FROM citiesWithManyHighConsHouses) "
					+"        UNION ALL "  // Note SQL command "UNION ALL" and not the SQL command "UNION"
					+"        (SELECT Country, 0 as CityWithManyHighConsHouses FROM Houses) "
					+"		) "
					+ "GROUP BY Country");


		highPwrConsCitiesPerCountryDF.write()
		.format("csv")
		.option("header", false)
		.save(outputPath2);

		// Close the Spark session
		ss.stop();
	}
}
