package it.polito.bigdata.spark.example;

import org.apache.spark.api.java.*;
import org.apache.spark.SparkConf;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import scala.Tuple2;

public class SparkDriver {
	
	public static void main(String[] args) {

		// The following two lines are used to switch off some verbose log messages
		Logger.getLogger("org").setLevel(Level.OFF);
		Logger.getLogger("akka").setLevel(Level.OFF);


		String housePath, consumptionPath;
		String outputPath1, outputPath2;

		housePath = args[0];
		consumptionPath = args[1];
		outputPath1 = args[2];
		outputPath2 = args[3];

	
		SparkConf conf=new SparkConf().setAppName("Exam20220202 Spark");
		
		// Use the following command to create the SparkConf object if you want to run
		// your application inside Eclipse.
		// Remember to remove .setMaster("local") before running your application on the cluster
		// SparkConf conf=new SparkConf().setAppName("Spark Lab #5").setMaster("local");
		
		JavaSparkContext sc = new JavaSparkContext(conf);

		JavaRDD<String> houseRDD = sc.textFile(housePath);
		JavaRDD<String> consumptionRDD = sc.textFile(consumptionPath);

		// Part 1
		// filter only the readings associated with year 2022
		JavaRDD<String> consumption2022 = consumptionRDD
											.filter(s -> {
			String[] fields = s.split(",");
			String date = fields[1];
			return date.startsWith("2022");
		});

		// compute the total amount of energy consumed in year 2022 for each house
		// key = houseID
		// value = kWh consumed in year 2022
		JavaPairRDD<String,Double> totalCons2022 = consumption2022
				.mapToPair(s -> {
			String[] fields = s.split(",");
			String hid = fields[0];
			Double consumption = Double.parseDouble(fields[2]);
			return new Tuple2<>(hid, consumption);
		})
				.reduceByKey((v1, v2) -> v1 + v2);

		// compute the avg power consumption per day
		// key = houseID
		// value = avg kWh consumed per day in year 2022
		// and filter only those with high avg consumption
		JavaPairRDD<String, Double> highAvgDailyCons = totalCons2022
														.mapValues(v -> v / 365)
														.filter(i -> i._2() > 30);

		// compute the pairRDD house -> country
		// key = houseID
		// value = country
		JavaPairRDD<String, String> houseCountry = houseRDD.mapToPair(s -> {
			String[] fields = s.split(",");
			String hid = fields[0];
			String country = fields[2];
			return new Tuple2<>(hid, country);
		});


		// keep an RDD containing countries with at least one house with high avg power consumption
		JavaRDD<String> countriesWithHighAvgPwrConsHouses = houseCountry
															.join(highAvgDailyCons)
															.map(it -> it._2()._1()); 

		// compute an RDD with all the countries
		// and subtract the countries with at least one house with high avg power consumption
		JavaRDD<String> res1 = houseCountry
								.map(v -> v._2()) 
								.distinct()
								.subtract(countriesWithHighAvgPwrConsHouses);

		res1.saveAsTextFile(outputPath1);

		// Part 2
		// keep only the houses for which the total power consumption over 2021 is > threshold kWh
		JavaPairRDD<String, Double> highTotalPowerCons2021 = consumptionRDD
																.filter(s -> {
														String[] fields = s.split(",");
														String date = fields[1];
														return date.startsWith("2021");
														})
														.mapToPair(s -> {
															String[] fields = s.split(",");
															String hid = fields[0];
															Double consumption = Double.parseDouble(fields[2]);
															return new Tuple2<>(hid, consumption);
														})
																.reduceByKey((v1, v2) -> v1 + v2)
																.filter(v -> v._2() > 10000);

		// compute an RDD with
		// key = houseID
		// value = (country, city)
		JavaPairRDD<String, Tuple2<String, String>> citiesRDD = houseRDD
				.mapToPair(s -> {
					String[] fields = s.split(",");
					String hid = fields[0];
					String city = fields[1];
					String country = fields[2];

					return new Tuple2<>(hid, new Tuple2<>(country, city));
				});

		// join the two RDDs and count for each city the number of houses with high annual power consumption
		// and filter only those cities with value > 500
		// key = (country, city)
		// value = #houses with high power consumption
		JavaPairRDD<Tuple2<String, String>, Integer> highPwrConsHousesPerCity = highTotalPowerCons2021
																					.join(citiesRDD)
																					.mapToPair(it -> new Tuple2<>(it._2()._2(), 1))
																					.reduceByKey((v1, v2) -> v1 + v2)
																					.filter(it -> it._2() > 500);

		// count for each country the number of cities with at least 500 houses with high annual power consumption
		// key = country
		// value = number of cities with at least 500 houses with high power consumption
		JavaPairRDD<String, Integer> highPwrConsCitiesPerCountry = highPwrConsHousesPerCity
																	.mapToPair(it -> new Tuple2<>(it._1()._1(), 1))
																	.reduceByKey((v1, v2) -> v1 + v2);


		JavaPairRDD<String, Integer> countries = houseCountry
												.mapToPair(it -> new Tuple2<>(it._2(), 0))
												.distinct();


		// add with a rightOuterJoin the countries with count == 0
		JavaPairRDD<String, Integer> res2 = highPwrConsCitiesPerCountry
												.rightOuterJoin(countries)
												.mapValues(it -> it._1().orElse(0));

		res2.saveAsTextFile(outputPath2);
		sc.close();
	}
}
